define([], function () {
    var orderingUtilService = function ($rootScope, localStorageService, moment, restService, itemPriceFilter) {
        var self = this;

        // Save it in memory as a singleton so it can be shared in more than one controller at a time
        var order = localStorageService.retrieve('order') || {};
        this.getOrder = function () {
            return order;
        };

        this.setOrder = function (data) {
            order = data;
            localStorageService.store('order', data);
        };

        this.clearOrder = function () {
            _.forEach(order, function (value, key) {
                order[key] = undefined;
            });
            localStorageService.remove('order');
        };

        var _calculateDiscount = function (type, price, discount) {
            if (type === 'value')
                return Math.min(price, discount);
            else if (type === 'percentage')
                return Math.min(price, price * discount / 100);
        };

        this.calculateDiscount = function (discount) {
            if (discount.target === 'total_cart') {
                return _calculateDiscount(discount.type, self.getTotalCart(), discount.amount);
            } else if (discount.target === 'items') {
                return _.sumBy(order.cart, function (item) {
                    var value = 0;
                    var itemSelected = item.selected;
                    if (_.includes(discount.items, itemSelected.variant.sku)) {
                        value += _calculateDiscount(discount.type, itemSelected.variant.price, discount.amount);
                    }
                    var addonsSelected = _.flatMap(itemSelected.addon_groups, 'addons');
                    _.forEach(addonsSelected, function (addonSelected) {
                        if (_.includes(discount.items, addonSelected.variant.sku)) {
                            value += _calculateDiscount(discount.type, addonSelected.variant.price, discount.amount);
                        }
                    });
                    value *= itemSelected.quantity;
                    return value;
                });
            } else if (discount.target === 'menus') {
                // get items that need to be discounted
                var filteredItems = _.filter(order.cart, function (item) {
                    return _.includes(discount.menus, item.menu_id);
                });
                // get total price for each item
                var totalPrice = _.sum(
                    _.map(filteredItems, function (item) {
                        var itemValue = item.selected.variant.price;
                        var addonsSelected = _.flatMap(item.selected.addon_groups, 'addons');
                        var addonsValue = _.sum(_.map(addonsSelected, function (addonSelected) {
                            return addonSelected.variant.price;
                        }));
                        var value = (itemValue + addonsValue) * item.selected.quantity;
                        return value;
                    })
                );
                return _calculateDiscount(discount.type, totalPrice, discount.amount);
            } else if (discount.target == 'delivery_fee') {
                var delivery_fee = _.get(_.find(order.pre_total_gross, { type: 'delivery_fee' }), 'amount', 0);
                return _calculateDiscount(discount.type, delivery_fee, discount.amount);
            } else {
                return 0;
            }
        };

        this.getTotalCart = function () {
            return Math.max(_.sumBy(order.cart, function (item) {
                return itemPriceFilter(item);
            }), 0);
        };

        this.getTotalGross = function () {
            return Math.max(self.getTotalCart(order) + _.sumBy(order.pre_total_gross, function (m) {
                return m.amount || m.amountFn();
            }), 0);
        };

        this.getTotalNet = function () {
            return Math.max(self.getTotalGross(order) + _.sumBy(order.pre_total_net, function (m) {
                return m.amount || m.amountFn();
            }), 0);
        };

        this.getTotalGrossPlusSomeTypes = function (order, types) {
            var filteredPreTotalNet = _.filter(order.pre_total_net, function (m) {
                return _.includes(types, m.type);
            });
            return Math.max(self.getTotalGross(order) + _.sumBy(filteredPreTotalNet, function (m) {
                return m.amount || m.amountFn();
            }), 0);
        };

        this.clearPreTotalGross = function (order) {
            order.pre_total_gross = [];
        };

        this.addPreTotalGross = function (order, modifier) {
            if (!order.pre_total_gross) {
                order.pre_total_gross = [];
            }
            order.pre_total_gross = _.concat(order.pre_total_gross, modifier);
        };

        this.clearPreTotalNet = function (order) {
            order.pre_total_net = [];
        };

        this.addPreTotalNet = function (order, modifier) {
            if (!order.pre_total_net) {
                order.pre_total_net = [];
            }
            order.pre_total_net = _.concat(order.pre_total_net, modifier);
        };

        this.removePreTotalNetByType = function (order, type) {
            _.remove(order.pre_total_net, { type: type });
        };

        this.removePreTotalNetById = function (order, id) {
            _.remove(order.pre_total_net, { id: id });
        };

        this.findPreTotalNetByType = function (order, type) {
            return !!_.find(order.pre_total_net, function (m) { return m.type === type; });
        };

        // Used for CATERING
        var stashedOrders = localStorageService.retrieve('stashedOrders') || [];
        this.getStashedOrders = function () {
            return stashedOrders;
        };

        this.stashOrder = function (data) {
            stashedOrders.push(data);
            localStorageService.store('stashedOrders', stashedOrders);
        };

        this.removeStashOrder = function (data) {
            _.remove(stashedOrders, function (item) {
                return moment(data.time).isSame(item.time);
            });
            localStorageService.store('stashedOrders', stashedOrders);
        };

        this.clearStashOrders = function () {
            stashedOrders = [];
            localStorageService.remove('stashedOrders');
        };

        $rootScope.$on('user:logout', function () {
            self.clearOrder();
        });
    };

    orderingUtilService.$inject = ['$rootScope', 'localStorageService', 'moment', 'restService', 'itemPriceFilter'];
    return orderingUtilService;
});
