define([], function () {
    var userService = function ($auth, $rootScope, restService, localStorageService, fencingService, frameworkService, utilityService, response_enum) {

        var updateLoggedInUser = function (data) {
            $rootScope.loggedInUser = angular.copy(data); // response.data might be modified by a controller after
            $rootScope.isLoggedIn = true;
        };

        var onLogin = function (response) {
            if (response.response_code === response_enum.CODE.SUCCESS && !_.isNil(response.data.username)) {
                updateLoggedInUser(response.data);
                $rootScope.$emit('user:login', response.data);
            }
            return response;
        };

        var onLogout = function (response) {
            $auth.logout();

            $rootScope.loggedInUser = null;
            $rootScope.isLoggedIn = false;
            $rootScope.$emit('user:logout');
            return response;
        };

        var onUpdate = function (response) {
            if (response.response_code === response_enum.CODE.SUCCESS) {
                updateLoggedInUser(response.data);
            }
            return response;
        };

        this.getUUID = function () {
            return localStorageService.retrieve('uuid');
        };

        this.setUUID = function (uuid) {
            localStorageService.store('uuid', uuid);
        };

        this.isAuthenticated = function () {
            return $auth.isAuthenticated();
        };

        this.signInWithOAuth = function (provider) {
            return $auth.authenticate(provider).then(onLogin);
        };

        this.linkWithOAuth = function (provider) {
            return $auth.link(provider).then(onUpdate);
        };

        this.unlinkFromOAuth = function (provider) {
            return $auth.unlink(provider).then(onUpdate);
        };

        this.getLoggedInUser = function () {
            var apiUrl = '/user';
            return restService.get(apiUrl)
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        updateLoggedInUser(response.data);
                    }
                    return response;
                });
        };

        this.login = function (username, password) {
            var data = {
                username: username,
                password: password,
                os: frameworkService.getPlatform(),
                uuid: this.getUUID()
            };
            return $auth.login(data)
                .then(onLogin);
        };

        this.logout = function () {
            // Remove the UUID from the server so the user doesn't receive notification
            var apiUrl = '/user/logout';
            var data = {
                os: frameworkService.getPlatform(),
                uuid: this.getUUID()
            };
            return restService.post(apiUrl, data)
                .then(onLogout, onLogout);
        };

        this.register = function (user) {
            return $auth.signup(user);
        };

        this.saveReferralCode = function (referralCode) {
            var apiUrl = '/user/referral';
            return restService.post(apiUrl, { referral_code: referralCode });
        };

        this.getActivities = function (page_size, page_number) {
            var apiUrl = '/user/activity/all/' + utilityService.getCurrency(fencingService.getCountry()) + '/' + page_size + '/' + page_number;
            return restService.get(apiUrl);
        };

        this.update = function (user) {
            var apiUrl = '/external/clique/user/details';
            return restService.put(apiUrl, user).then(onUpdate);
        };

        this.updateUsername = function (token) {
            var apiUrl = '/external/clique/user/username';
            return restService.post(apiUrl, { token: token }).then(onUpdate);
        };

        this.sendForgotPassword = function (username) {
            var apiUrl = '/external/clique/user/forgotpassword';
            return restService.post(apiUrl, { username: username });
        };

        this.resetPassword = function (user) {
            var apiUrl = '/external/clique/user/resetpassword';
            return restService.put(apiUrl, user);
        };

        this.changePassword = function (user) {
            var apiUrl = '/external/clique/user/changepassword';
            return restService.put(apiUrl, user);
        };

        this.getCliqueLocations = function () {
            var apiUrl = '/external/clique/location/all';
            return restService.get(apiUrl);
        };

        this.activate = function (token) {
            var apiUrl = '/external/clique/user/activate';
            return restService.post(apiUrl, { token: token })
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        $auth.setToken(response.data.token);
                    }
                    return response;
                })
                .then(onLogin);
        };

        this.activateFacebookUser = function (token) {
            var apiUrl = '/external/clique/user/activate/facebook';
            return restService.post(apiUrl, { token: token })
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        $auth.setToken(response.data.token);
                    }
                    return response;
                })
                .then(onLogin);
        };

        this.resendActivateUser = function (username) {
            var apiUrl = '/external/clique/user/activate/resend';
            return restService.post(apiUrl, { username: username });
        };

        this.completeRegistration = function (user) {
            var apiUrl = '/external/clique/user/register/complete';
            return restService.post(apiUrl, user);
        };

        this.getKids = function () {
            var apiUrl = '/external/clique/user/kid';
            return restService.get(apiUrl);
        };

        this.saveKids = function (kids) {   // add and edit kid in one call
            var apiUrl = '/external/clique/user/kid';
            return restService.post(apiUrl, kids);
        };

        this.getRegistrationDetails = function (username) {
            var apiUrl = '/external/clique/user/registrationdetails/' + username;
            return restService.get(apiUrl);
        };
    };

    userService.$inject = ['$auth', '$rootScope', 'restService', 'localStorageService', 'fencingService', 'frameworkService', 'utilityService', 'response_enum'];
    return userService;
});
