define(['qrcode'], function (qrcode) {
    var utilityService = function ($q, moment, frameworkService, country_options, response_enum) {
        this.createFetcher = function (page_size, retrieve) {
            return {
                page_size: page_size,
                page_number: 0,
                total_pages: Infinity,
                total_records: Infinity,
                current_promise: $q.when(null),
                next: function () {
                    var self = this;

                    if (self.page_number < self.total_pages) {
                        self.page_number++;

                        self.current_promise = retrieve(self.page_size, self.page_number)
                            .then(function (response) {
                                if (response.response_code === response_enum.CODE.SUCCESS) {
                                    self.total_pages = response.pagination.total_pages;
                                    self.total_records = response.pagination.total_records;
                                    return response;
                                } else {
                                    frameworkService.showError(response.data);
                                    return $q.reject();
                                }
                            });
                    } else {
                        self.current_promise = $q.when(null);
                    }
                    return self.current_promise;
                },
                reset: function () {
                    var self = this;
                    return self.current_promise.then(function () { // Wait for the current retrieve promise to finish before resetting
                        self.page_number = 0;
                        self.total_pages = Infinity;
                    });
                }
            };
        };

        this.createEmptyFetcher = function () {
            return {
                next: function () {
                    return $q.when(null);
                },
                reset: function () { }
            };
        };

        this.processResponse = function (method) {
            return method()
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        return response;
                    } else {
                        frameworkService.showError(response.data);
                        return $q.reject();
                    }
                });
        };

        // Works like String.format() of C#
        this.stringFormat = function (format) {
            var args = Array.prototype.slice.call(arguments, 1);
            return format.replace(/{(\d+)}/g, function (match, number) {
                return typeof args[number] != 'undefined' ? args[number] : undefined;
            });
        };

        this.generateQRcode = function (selector_id, value, width, height) {
            return new QRCode(selector_id, {
                text: value,
                width: width || 150,
                height: height || 150,
                colorDark: '#000000',
                colorLight: 'transparent',
                correctLevel: QRCode.CorrectLevel.H
            });
        };

        this.clearQRcode = function (selector_id) {
            angular.element(document.querySelector('#' + selector_id)).empty();
        };

        var days = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'];
        this.groupTimeIntervalsByDay = function (time_intervals) {
            var ans = {};
            _.forEach(days, function (d) {
                ans[d] = _.filter(time_intervals, function (item) {
                    return item.day === d;
                });
            });
            return ans;
        };

        this.getCurrency = function (country) {
            return _.get(_.find(country_options, { _id: country }), 'currency');
        };

        this.getCountryCode = function (country) {
            return _.get(_.find(country_options, { _id: country }), 'code');
        };

        this.getDayName = function (day, timezone) {
            var todayNumber = moment.tz(day, timezone).isoWeekday();
            return days[todayNumber - 1];
        };

        // Time is in the format HH:mm
        this.setTime = function (day, time, timezone) {
            return moment.tz(day, timezone).set({
                hour: time.split(':')[0],
                minute: time.split(':')[1]
            });
        };

        this.getParameterByName = function (name, url) {
            name = name.replace(/[\[\]]/g, '\\$&');
            var regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)'),
                results = regex.exec(url);
            if (!results) return null;
            if (!results[2]) return '';
            return decodeURIComponent(results[2].replace(/\+/g, ' '));
        };
    };


    utilityService.$inject = ['$q', 'moment', 'frameworkService', 'country_options', 'response_enum'];
    return utilityService;
});
